import { Directive, ElementRef, Host, HostBinding, HostListener, Input, NgModule } from '@angular/core';
import { Subject as Subject$1 } from 'rxjs/Subject';
import 'rxjs/add/operator/takeUntil';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import 'rxjs/add/operator/filter';

class ContentsDirective {
    constructor() {
        this._onScroll$ = new Subject$1();
        this._activeSection$ = new BehaviorSubject$1(null);
        this.scrollFun = (event) => this._onScroll$.next(event);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.unsubscribeScrollEventListener();
        this.subscribeScrollEventListener();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.unsubscribeScrollEventListener();
        this.subscribeScrollEventListener();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unsubscribeScrollEventListener();
    }
    /**
     * @return {?}
     */
    subscribeScrollEventListener() {
        (this.scrollingView || document).addEventListener('scroll', this.scrollFun, false);
    }
    /**
     * @return {?}
     */
    unsubscribeScrollEventListener() {
        (this.scrollingView || document).removeEventListener('scroll', this.scrollFun, false);
    }
    /**
     * @return {?}
     */
    activeSection() {
        return this._activeSection$
            .asObservable()
            .filter(section => !!section); // Prevent returning null values.
    }
}
ContentsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contents]',
                exportAs: 'contents',
            },] },
];
/**
 * @nocollapse
 */
ContentsDirective.ctorParameters = () => [];
ContentsDirective.propDecorators = {
    'scrollingView': [{ type: Input },],
};

class ContentsLinkDirective {
    /**
     * @param {?} contents
     */
    constructor(contents) {
        this.contents = contents;
        this.ngUnsubscribe = new Subject$1();
        this.active = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.contents._activeSection$
            .takeUntil(this.ngUnsubscribe)
            .subscribe((sectionName) => {
            this.active = `#${sectionName}` === this.href;
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.ngUnsubscribe.next();
        this.ngUnsubscribe.complete();
    }
}
ContentsLinkDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contentsLink]',
                exportAs: 'contentsLink',
            },] },
];
/**
 * @nocollapse
 */
ContentsLinkDirective.ctorParameters = () => [
    { type: ContentsDirective, decorators: [{ type: Host },] },
];
ContentsLinkDirective.propDecorators = {
    'href': [{ type: Input },],
    'active': [{ type: HostBinding, args: ['class.active',] },],
};

/**
 * @param {?} element
 * @return {?}
 */
function getAbsoluteHeight(element) {
    const /** @type {?} */ styles = window.getComputedStyle(element);
    const /** @type {?} */ margin = parseFloat(styles.marginTop || '0') +
        parseFloat(styles.marginBottom || '0');
    return Math.ceil(element.offsetHeight + margin);
}
/**
 * @return {?}
 */
function documentOffset() {
    return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}

class ContentsSectionDirective {
    /**
     * @param {?} contents
     * @param {?} elementRef
     */
    constructor(contents, elementRef) {
        this.contents = contents;
        this.elementRef = elementRef;
        this.ngUnsubscribe = new Subject$1();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.detectActiveChanges();
        this.contents._onScroll$
            .takeUntil(this.ngUnsubscribe)
            .subscribe((event) => {
            this.detectActiveChanges();
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.ngUnsubscribe.next();
        this.ngUnsubscribe.complete();
    }
    /**
     * @return {?}
     */
    detectActiveChanges() {
        if (this.isInRange() || !this.contents._activeSection$.value) {
            this.contents._activeSection$.next(this.contentsSection);
        }
    }
    /**
     * @return {?}
     */
    isInRange() {
        const /** @type {?} */ pageOffset = this.contents.scrollingView ? this.contents.scrollingView.scrollTop : documentOffset();
        const /** @type {?} */ element = this.elementRef.nativeElement;
        const /** @type {?} */ offset = element.offsetTop;
        const /** @type {?} */ height = getAbsoluteHeight(element);
        return pageOffset >= offset && pageOffset <= offset + height;
    }
}
ContentsSectionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contentsSection]',
                exportAs: 'contentsSection',
            },] },
];
/**
 * @nocollapse
 */
ContentsSectionDirective.ctorParameters = () => [
    { type: ContentsDirective, decorators: [{ type: Host },] },
    { type: ElementRef, },
];
ContentsSectionDirective.propDecorators = {
    'contentsSection': [{ type: HostBinding, args: ['id',] }, { type: Input },],
};

class ContentsTableDirective {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.sticky = false;
        this.marginTop = '0px';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.updateStickiness();
    }
    /**
     * Check whether the Table of Contents should be a sticky, to keep itself visible while the user
     * scrolls.
     * @return {?}
     */
    updateStickiness() {
        const /** @type {?} */ pageOffset = window.pageYOffset;
        const /** @type {?} */ parentElement = this.elementRef.nativeElement.parentNode;
        const /** @type {?} */ parentOffset = parentElement.offsetTop;
        const /** @type {?} */ parentHeight = getAbsoluteHeight(parentElement);
        const /** @type {?} */ element = this.elementRef.nativeElement;
        const /** @type {?} */ elementInnerHeight = element.offsetHeight;
        if (pageOffset + elementInnerHeight > parentOffset + parentHeight) {
            // Use a fixed margin-top instead when scrolling past the parent container.
            this.sticky = false;
            this.marginTop = `${parentHeight - elementInnerHeight}px`;
        }
        else {
            // Use CSS sticky when scrolling into the parent container.
            this.marginTop = '0px';
            this.sticky = pageOffset > parentOffset;
        }
    }
    /**
     * @param {?} target
     * @return {?}
     */
    getHeight(target) {
        const /** @type {?} */ element = this.elementRef.nativeElement;
        const /** @type {?} */ styles = window.getComputedStyle(element);
        const /** @type {?} */ margin = parseFloat(styles.marginTop || '0') +
            parseFloat(styles.marginBottom || '0');
        return Math.ceil(element.offsetHeight + margin);
    }
}
ContentsTableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[contentsTable]',
                exportAs: 'contentsTable',
            },] },
];
/**
 * @nocollapse
 */
ContentsTableDirective.ctorParameters = () => [
    { type: ElementRef, },
];
ContentsTableDirective.propDecorators = {
    'sticky': [{ type: HostBinding, args: ['class.sticky',] },],
    'marginTop': [{ type: HostBinding, args: ['style.margin-top',] },],
    'updateStickiness': [{ type: HostListener, args: ['window:scroll', ['$event'],] },],
};

class ContentsModule {
}
ContentsModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [
                    ContentsLinkDirective,
                    ContentsSectionDirective,
                    ContentsTableDirective,
                    ContentsDirective,
                ],
                declarations: [
                    ContentsLinkDirective,
                    ContentsSectionDirective,
                    ContentsTableDirective,
                    ContentsDirective,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ContentsModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { ContentsLinkDirective, ContentsSectionDirective, ContentsTableDirective, ContentsDirective, ContentsModule };
//# sourceMappingURL=angular-contents.js.map
